VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "Theme"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

' Version number of Class
Dim myVersion As Integer
' Unique number for this Theme
Dim myIndex As Integer
' Name of Theme
Dim myName As String
' Comments for Theme
Dim myComments As String
' Maximum numer of Encounters covered by the Theme
Dim myCoverage As Integer
' Number of Encounters already covered by Theme
Dim myEncounterCount As Integer
' Bit1      - IsQuest? (Can span multiple maps) On=Yes, Off=No
' Bit2-3    - Not Used
' Bit4      - IsMajorStory? (Awards Skill Points) On=Yes, Off=No
' Bit5-7    - Not Used
Dim myFlags As Integer
' Created for Party Size
' Bit1-2 - 0 = Solo, 1 = 2-3, 2 = 4-6, 3 = 7-9
' With Average Party Level
' Bit3-4 - 0 = 1-3, 1 = 4-6, 2 = 7-10, 3 = 10+
' Bit5-8 (MapStyle)
'       0 - Town Map
'       1 - Wilderness Map
'       2 - Building Map
'       3 - Dungeon Map
' Bit9-16 - Not Used
Dim myStyle As Integer
' Total SkillPoints to value total Encounters built by this Theme
Dim mySkillPoints As Integer
' List of Encounters for Theme
Dim myEncounters As New Collection
' List of Creatures for Theme.
Dim myCreatures As New Collection
' List of Items for Theme.
Dim myItems As New Collection
' List of Triggers (usually Traps because they can appear anywhere).
Dim myTriggers As New Collection
' List of descriptions for this Theme. Applies to Encounter
' FirstEntry description only.
Dim myFactoids As New Collection

Public Property Get Name() As String
    Name = Trim(myName)
End Property

Public Property Let Name(NewValue As String)
    myName = Trim(NewValue)
End Property

Public Property Get Comments() As String
    Comments = Trim(myComments)
End Property

Public Property Let Comments(NewValue As String)
    myComments = Trim(NewValue)
End Property

Public Property Get Index() As Integer
    Index = myIndex
End Property

Public Property Let Index(NewValue As Integer)
    myIndex = NewValue
End Property

Public Property Get EncounterCount() As Integer
    EncounterCount = myEncounterCount
End Property

Public Property Let EncounterCount(NewValue As Integer)
    myEncounterCount = NewValue
End Property

Public Property Get Flags() As Integer
    Flags = myFlags
End Property

Public Property Let Flags(NewValue As Integer)
    myFlags = NewValue
End Property

Public Property Get Style() As Integer
    Style = myStyle
End Property

Public Property Let Style(NewValue As Integer)
    myStyle = NewValue
End Property

Public Property Get MapStyle() As Integer
    MapStyle = CInt((myStyle And &HF0) / 16)
End Property

Public Property Let MapStyle(NewValue As Integer)
    myStyle = myStyle And &HFF0F
    myStyle = myStyle Or (NewValue * 16 And &HF0)
End Property

Public Property Get PartySize() As Integer
    PartySize = CInt(myStyle And &H3)
End Property

Public Property Let PartySize(NewValue As Integer)
    myStyle = myStyle And &HFFFC
    myStyle = myStyle Or (NewValue And &H3)
End Property

Public Property Get PartyAvgLevel() As Integer
    PartyAvgLevel = CInt((myStyle And &HC) / 4)
End Property

Public Property Let PartyAvgLevel(NewValue As Integer)
    myStyle = myStyle And &HFFF3
    myStyle = myStyle Or (NewValue * 4 And &HC)
End Property

Public Property Get EncounterPoints() As Integer
    Dim c As Integer
    Select Case Me.PartyAvgLevel
        Case 0  ' 1-3
            c = 1
        Case 1  ' 4-6
            c = 3
        Case 2  ' 7-10
            c = 6
        Case 3  ' 10+
            c = 12
    End Select
    Select Case Me.PartySize
        Case 0  ' Solo
            c = 1 * c
        Case 1  ' 2-3
            c = 3 * c
        Case 2  ' 4-6
            c = 5 * c
        Case 3  ' 7-9
            c = 10 * c
    End Select
    ' Randomize (EncounterPoints vary each time you ask for them)
    Select Case Int(Rnd * 100)
        Case 0
            EncounterPoints = Int(c / 4)
        Case 1 To 5
            EncounterPoints = Int(c / 3)
        Case 6 To 15
            EncounterPoints = Int(c / 2)
        Case 16 To 25
            EncounterPoints = Int(c / 1.2)
        Case 26 To 75
            EncounterPoints = c
        Case 76 To 85
            EncounterPoints = Int(c * 1.2)
        Case 86 To 94
            EncounterPoints = Int(c * 2)
        Case 95 To 98
            EncounterPoints = Int(c * 3)
        Case 99
            EncounterPoints = Int(c * 4)
    End Select
End Property

Public Property Get IsQuest() As Integer
    IsQuest = (myFlags And &H1) > 0
End Property

Public Property Let IsQuest(NewValue As Integer)
    If NewValue Then
        myFlags = myFlags Or &H1
    Else
        myFlags = myFlags And (Not &H1)
    End If
End Property

Public Property Get IsMajorTheme() As Integer
    IsMajorTheme = (myFlags And &H8) > 0
End Property

Public Property Let IsMajorTheme(NewValue As Integer)
    If NewValue Then
        myFlags = myFlags Or &H8
    Else
        myFlags = myFlags And (Not &H8)
    End If
End Property

Public Property Get Coverage() As Integer
    Coverage = myCoverage
End Property

Public Property Let Coverage(NewValue As Integer)
    myCoverage = NewValue
End Property

Public Property Get SkillPoints() As Integer
    SkillPoints = mySkillPoints
End Property

Public Property Let SkillPoints(NewValue As Integer)
    mySkillPoints = NewValue
End Property

Public Property Get Encounters() As Collection
    Set Encounters = myEncounters
End Property

Public Function AddEncounter() As Encounter
    ' Adds an Encounter and returns a reference to that Encounter
    Dim c As Integer
    Dim EncounterX As Encounter
    ' Find new available unused index idenifier
    c = 1
    For Each EncounterX In myEncounters
        If EncounterX.Index >= c Then
            c = EncounterX.Index + 1
        End If
    Next EncounterX
    ' Set the index and add the encounter. Return the new encounter's index.
    Set EncounterX = New Encounter
    EncounterX.Index = c
    EncounterX.Name = "Encounter" & c
    EncounterX.ParentTheme = myIndex
    myEncounters.Add Item:=EncounterX, Key:="E" & EncounterX.Index
    Set AddEncounter = EncounterX
End Function

Public Sub RemoveEncounter(DeleteKey As String)
    myEncounters.Remove DeleteKey
End Sub

Public Property Get Creatures() As Collection
    Set Creatures = myCreatures
End Property

Public Function AddCreature() As Creature
    ' Adds an Creature and returns a reference to that Creature
    Dim c As Integer
    Dim CreatureX As Creature
    ' Find new available unused index idenifier
    c = 1
    For Each CreatureX In myCreatures
        If CreatureX.Index >= c Then
            c = CreatureX.Index + 1
        End If
    Next CreatureX
    ' Set the index and add the Creature. Return the new Creature's index.
    Set CreatureX = New Creature
    CreatureX.Index = c
    CreatureX.Name = "Creature" & c
    myCreatures.Add Item:=CreatureX, Key:="X" & CreatureX.Index
    Set AddCreature = CreatureX
End Function

Public Sub RemoveCreature(DeleteKey As String)
    myCreatures.Remove DeleteKey
End Sub

Public Property Get Items() As Collection
    Set Items = myItems
End Property

Public Function AddItem() As Item
    ' Adds an Item and returns a reference to that Item
    Dim c As Integer
    Dim ItemX As Item
    ' Find new available unused index idenifier
    c = 1
    For Each ItemX In myItems
        If ItemX.Index >= c Then
            c = ItemX.Index + 1
        End If
    Next ItemX
    ' Set the index and add the Item. Return the new Item's index.
    Set ItemX = New Item
    ItemX.Index = c
    ItemX.Name = "Item" & c
    myItems.Add Item:=ItemX, Key:="I" & ItemX.Index
    Set AddItem = ItemX
End Function

Public Sub RemoveItem(DeleteKey As String)
    myItems.Remove DeleteKey
End Sub

Public Property Get Triggers() As Collection
    Set Triggers = myTriggers
End Property

Public Function AddTrigger() As Trigger
    ' Adds an Trigger and returns a reference to that Trigger
    Dim c As Integer
    Dim TriggerX As Trigger
    ' Find new available unused index idenifier
    c = 1
    For Each TriggerX In myTriggers
        If TriggerX.Index >= c Then
            c = TriggerX.Index + 1
        End If
    Next TriggerX
    ' Set the index and add the Trigger. Return the new Trigger's index.
    Set TriggerX = New Trigger
    TriggerX.Index = c
    TriggerX.Name = "Trigger" & c
    myTriggers.Add Item:=TriggerX, Key:="T" & TriggerX.Index
    Set AddTrigger = TriggerX
End Function

Public Sub RemoveTrigger(DeleteKey As String)
    myTriggers.Remove DeleteKey
End Sub

Public Property Get Factoids() As Collection
    Set Factoids = myFactoids
End Property

Public Function AddFactoid() As Factoid
    ' Adds an Description and returns a reference to that Description
    Dim c As Integer
    Dim FactoidX As Factoid
    ' Find new available unused index idenifier
    c = 1
    For Each FactoidX In myFactoids
        If FactoidX.Index >= c Then
            c = FactoidX.Index + 1
        End If
    Next FactoidX
    ' Set the index and add the Description. Return the new Description's index.
    Set FactoidX = New Factoid
    FactoidX.Index = c
    myFactoids.Add Item:=FactoidX, Key:="F" & FactoidX.Index
    Set AddFactoid = FactoidX
End Function

Public Sub RemoveFactoid(DeleteKey As String)
    myFactoids.Remove DeleteKey
End Sub

Public Sub Copy(FromTheme As Theme)
    Dim c As Integer
    Dim myEncounter As Encounter, myItem As Item
    Dim myCreature As Creature, myTrigger As Trigger
    Dim myFactoid As Factoid
    ' Copy Theme Name
    myName = FromTheme.Name
    myComments = FromTheme.Comments
    myCoverage = FromTheme.Coverage
    mySkillPoints = FromTheme.SkillPoints
    myStyle = FromTheme.Style
    myFlags = FromTheme.Flags
    ' Copy Encounters for Theme
    Set myEncounters = New Collection
    For c = 1 To FromTheme.Encounters.Count
        Set myEncounter = Me.AddEncounter
        myEncounter.Copy FromTheme.Encounters(c)
    Next c
    ' Copy Creatures for Theme
    Set myCreatures = New Collection
    For c = 1 To FromTheme.Creatures.Count
        Set myCreature = Me.AddCreature
        myCreature.Copy FromTheme.Creatures(c)
    Next c
    ' Copy Items for Theme
    Set myItems = New Collection
    For c = 1 To FromTheme.Items.Count
        Set myItem = Me.AddItem
        myItem.Copy FromTheme.Items(c)
    Next c
    ' Copy Triggers for Theme
    Set myTriggers = New Collection
    For c = 1 To FromTheme.Triggers.Count
        Set myTrigger = Me.AddTrigger
        myTrigger.Copy FromTheme.Triggers(c)
    Next c
    ' Copy Descriptions for Theme
    Set myFactoids = New Collection
    For c = 1 To FromTheme.Factoids.Count
        Set myFactoid = Me.AddFactoid
        myFactoid.Copy FromTheme.Factoids(c)
    Next c
End Sub

Public Sub ReadFromFileHeader(FromFile As Integer)
    Dim c As Integer, Cnt As Long
    ' Read Theme Name
    Get FromFile, , myVersion
    Get FromFile, , Cnt
    myName = ""
    For c = 1 To Cnt
        myName = myName & " "
    Next c
    Get FromFile, , myName
    ' Read Theme Comments
    Get FromFile, , Cnt
    myComments = ""
    For c = 1 To Cnt
        myComments = myComments & " "
    Next c
    Get FromFile, , myComments
    Get FromFile, , myCoverage
    Get FromFile, , myIndex
    Get FromFile, , mySkillPoints
    Get FromFile, , myStyle
    Get FromFile, , myFlags
End Sub

Public Sub ReadFromFile(FromFile As Integer)
    Dim c As Integer, Cnt As Long
    Dim myEncounter As Encounter, myItem As Item
    Dim myCreature As Creature, myTrigger As Trigger
    Dim myFactoid As Factoid
    ReadFromFileHeader FromFile
    ' Read Encounters for Theme
    Get FromFile, , Cnt
    For c = 1 To Cnt
        Set myEncounter = New Encounter
        myEncounter.ReadFromFile FromFile
        myEncounters.Add Item:=myEncounter, Key:="E" & myEncounter.Index
    Next c
    ' Read Creatures for Theme
    Get FromFile, , Cnt
    For c = 1 To Cnt
        Set myCreature = New Creature
        myCreature.ReadFromFile FromFile
        myCreatures.Add Item:=myCreature, Key:="X" & myCreature.Index
    Next c
    ' Read Items for Theme
    Get FromFile, , Cnt
    For c = 1 To Cnt
        Set myItem = New Item
        myItem.ReadFromFile FromFile
        myItems.Add Item:=myItem, Key:="I" & myItem.Index
    Next c
    ' Read Triggers for Theme
    Get FromFile, , Cnt
    For c = 1 To Cnt
        Set myTrigger = New Trigger
        myTrigger.ReadFromFile FromFile
        myTriggers.Add Item:=myTrigger, Key:="T" & myTrigger.Index
    Next c
    ' Read Descriptions for Theme
    Get FromFile, , Cnt
    For c = 1 To Cnt
        Set myFactoid = New Factoid
        myFactoid.ReadFromFile FromFile
        myFactoids.Add Item:=myFactoid, Key:="F" & myFactoid.Index
    Next c
End Sub

Public Sub SaveToFile(ToFile As Integer)
    Dim c As Integer
    ' Save Theme Name
    Put ToFile, , myVersion
    Put ToFile, , Len(myName)
    Put ToFile, , myName
    Put ToFile, , Len(myComments)
    Put ToFile, , myComments
    Put ToFile, , myCoverage
    Put ToFile, , myIndex
    Put ToFile, , mySkillPoints
    Put ToFile, , myStyle
    Put ToFile, , myFlags
    ' Save Encounters for Theme
    Put ToFile, , myEncounters.Count
    For c = 1 To myEncounters.Count
        myEncounters(c).SaveToFile ToFile
    Next c
    ' Save Creatures for Theme
    Put ToFile, , myCreatures.Count
    For c = 1 To myCreatures.Count
        myCreatures(c).SaveToFile ToFile
    Next c
    ' Save Items for Theme
    Put ToFile, , myItems.Count
    For c = 1 To myItems.Count
        myItems(c).SaveToFile ToFile
    Next c
    ' Save Triggers for Theme
    Put ToFile, , myTriggers.Count
    For c = 1 To myTriggers.Count
        myTriggers(c).SaveToFile ToFile
    Next c
    ' Save Descriptions for Theme
    Put ToFile, , myFactoids.Count
    For c = 1 To myFactoids.Count
        myFactoids(c).SaveToFile ToFile
    Next c
End Sub

Private Sub Class_Initialize()
    myName = "Unnamed"
    Me.PartySize = 2
    Me.PartyAvgLevel = 0
    Me.Coverage = 1
End Sub
