VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "Tome"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit
' This file is used to identify a group of Plots. It also serves as
' the Party holder --- holding a collection of Creatures (CreatureWithTurns, Story
' CreatureWithTurns and NCreatureWithTurns) which make up the Party.

' Version number of this Class
Private myVersion As Integer
' Index used in TomeList (not saved)
Private myIndex As Integer
' The Name as it will appear in the Bookmarks.
Private myName As String
' Used only at Design Time.
Private myComments As String
' Tome's home directory
Private myFullPath As String
' Directory where Creator loaded from
Private myLoadPath As String
' FileName (not saved)
Private myFileName As String

' Name of Plot (.rsa file) to jump to when hit this entry point
Dim myAreaIndex As Integer
' Name of Map within above Plot to jump to.
Dim myMapIndex As Integer
' Name of EntryPoint within above Map.
Dim myEntryIndex As Integer

' (X,Y) of Party on Plot.Map (not currently used)
Private myMapX As Integer
Private myMapY As Integer
' This is run-time only data
Private myMoveToX As Integer
Private myMoveToY As Integer
' Que of Steps from A* routine
' Run-time only data
Private myStepX() As Integer
Private myStepY() As Integer
Private myMaxStep As Integer
Private myNextStep As Integer
Private myLastEncounter As Integer
Private myCurrEncounter As Integer
Private myFleeToX As Integer
Private myFleeToY As Integer

' Current Turn, Cycle, Moon, Year
Private myTurn As Byte      ' 200 Turns per Cycle
Private myCycle As Byte     ' 50 Cycles per Moon
Private myMoon As Byte      ' 10 Moons per Year
Private myYear As Integer   ' 10000 Years per Age
' Real Seconds, Minutes, Hours, Days
Private myRealSeconds As Integer
Private myRealMinutes As Integer
Private myRealHours As Integer
Private myRealDays As Integer

' Marked as True if this Tome needs to be saved (not saved)
Private myDirty As Integer
' Used while loading up the main menu (not saved)
Private myInWorld As Integer

' Bit0      -   On=OnAdventure, Off=Not OnAdventure
' Bit1      -   On=IsReset, Off=Not Reset
' Bit2      -   On=IsInPlay, Off=Not InPlay
' Bit3-7    -   Not Used
Private myOnAdventure As Byte

' Max number of Creatures in the Party. 0 is no max.
Private myPartySize As Byte
' Max amount of Level in Party. 0 is no max.
Private myPartyAvgLevel As Byte

' Collection of Creature Objects which make up the Party. Upon arriving
' at MainMenu, this is emptied of all CreatureWithTurns (they are saved back into
' the Pool).
Private myCreatures As New Collection
' Triggers which follow with Party. Upon arriving at MainMenu, this
' is cleared.
Private myTriggers As New Collection
' Journal Entries attached to this Tome
Private myJournals As New Collection
' This is the list of Factoids accquired by the Party.
Private myFactoids As New Collection
' Areas attached to this Tome
Private myAreas As New Collection

Public Property Get Index() As Integer
    Index = myIndex
End Property

Public Property Let Index(NewValue As Integer)
    myIndex = NewValue
End Property

Public Property Get Name() As String
    Name = Trim(myName)
End Property

Public Property Let Name(NewName As String)
    myName = Trim(NewName)
End Property

Public Property Get FullPath() As String
    FullPath = Trim(myFullPath)
End Property

Public Property Let FullPath(NewFullPath As String)
    myFullPath = Trim(NewFullPath)
End Property

Public Property Get LoadPath() As String
    LoadPath = Trim(myLoadPath)
End Property

Public Property Let LoadPath(NewLoadPath As String)
    myLoadPath = Trim(NewLoadPath)
End Property

Public Property Get FileName() As String
    FileName = Trim(myFileName)
End Property

Public Property Let FileName(NewFileName As String)
    myFileName = Trim(NewFileName)
End Property

Public Property Get Comments() As String
    Comments = Trim(myComments)
End Property

Public Property Let Comments(NewComments As String)
    myComments = Trim(NewComments)
End Property

Public Property Get AreaIndex() As Integer
    AreaIndex = myAreaIndex
End Property

Public Property Let AreaIndex(NewValue As Integer)
    myAreaIndex = NewValue
End Property

Public Property Get MapIndex() As Integer
    MapIndex = myMapIndex
End Property

Public Property Let MapIndex(NewValue As Integer)
    myMapIndex = NewValue
End Property

Public Property Get EntryIndex() As Integer
    EntryIndex = myEntryIndex
End Property

Public Property Let EntryIndex(NewValue As Integer)
    myEntryIndex = NewValue
End Property

Public Property Get MapX() As Integer
    MapX = myMapX
End Property

Public Property Let MapX(NewMapX As Integer)
    myMapX = NewMapX
End Property

Public Property Get MapY() As Integer
    MapY = myMapY
End Property

Public Property Let MapY(NewMapY As Integer)
    myMapY = NewMapY
End Property

Public Property Get FleeToX() As Integer
    FleeToX = myFleeToX
End Property

Public Property Let FleeToX(NewValue As Integer)
    myFleeToX = NewValue
End Property

Public Property Get FleeToY() As Integer
    FleeToY = myFleeToY
End Property

Public Property Let FleeToY(NewValue As Integer)
    myFleeToY = NewValue
End Property

Public Property Get InWorld() As Integer
    InWorld = myInWorld
End Property

Public Property Let InWorld(NewValue As Integer)
    myInWorld = NewValue
End Property

Public Sub AddStep(NewX As Integer, NewY As Integer)
    Dim c As Integer
    ' Preserve previous steps
    ReDim KeepX(myMaxStep) As Integer
    ReDim KeepY(myMaxStep) As Integer
    If myMaxStep > 0 Then
        For c = 0 To myMaxStep
            KeepX(c) = myStepX(c)
            KeepY(c) = myStepY(c)
        Next c
    End If
    ' Resize for adding a step
    myMaxStep = myMaxStep + 1
    ReDim myStepX(myMaxStep) As Integer
    ReDim myStepY(myMaxStep) As Integer
    ' Replace previous steps
    For c = 0 To myMaxStep - 1
        myStepX(c) = KeepX(c)
        myStepY(c) = KeepY(c)
    Next c
    ' Add new step
    myStepX(myMaxStep) = NewX
    myStepY(myMaxStep) = NewY
End Sub

Public Sub NextStep(ByRef x As Integer, ByRef y As Integer)
    Dim XDir As Integer, YDir As Integer, c As Integer
    Dim CreatureX As Creature
    If myMaxStep > 0 Then
        c = 0
        XDir = myStepX(myMaxStep) - myMapX
        YDir = myStepY(myMaxStep) - myMapY
        For Each CreatureX In myCreatures
            If XDir < 0 Then ' Moving Down Left
                Select Case CreatureX.TileSpot
                    Case 0, 1, 3, 4, 6, 7
                        CreatureX.TileSpot = CreatureX.TileSpot + 1
                    Case Else
                        CreatureX.TileSpot = CreatureX.TileSpot - 2
                        CreatureX.MapX = myStepX(myMaxStep): CreatureX.MapY = myStepY(myMaxStep)
                End Select
            ElseIf XDir > 0 Then    ' Moving Up Right
                Select Case CreatureX.TileSpot
                    Case 1, 2, 4, 5, 7, 8
                        CreatureX.TileSpot = CreatureX.TileSpot - 1
                    Case Else
                        CreatureX.TileSpot = CreatureX.TileSpot + 2
                        CreatureX.MapX = myStepX(myMaxStep): CreatureX.MapY = myStepY(myMaxStep)
                End Select
            ElseIf YDir < 0 Then    ' Moving Up Left
                Select Case CreatureX.TileSpot
                    Case 3, 4, 5, 6, 7, 8
                        CreatureX.TileSpot = CreatureX.TileSpot - 3
                    Case Else
                        CreatureX.TileSpot = CreatureX.TileSpot + 6
                        CreatureX.MapX = myStepX(myMaxStep): CreatureX.MapY = myStepY(myMaxStep)
                End Select
            ElseIf YDir > 0 Then    ' Moving Down Right
                Select Case CreatureX.TileSpot
                    Case 0, 1, 2, 3, 4, 5
                        CreatureX.TileSpot = CreatureX.TileSpot + 3
                    Case Else
                        CreatureX.TileSpot = CreatureX.TileSpot - 6
                        CreatureX.MapX = myStepX(myMaxStep): CreatureX.MapY = myStepY(myMaxStep)
                End Select
            End If
            If CreatureX.MapX = myStepX(myMaxStep) And CreatureX.MapY = myStepY(myMaxStep) Then
                c = c + 1
            End If
        Next CreatureX
        If c = myCreatures.Count Then
            x = myStepX(myMaxStep): y = myStepY(myMaxStep)
            myMaxStep = myMaxStep - 1
        Else
            x = myMapX: y = myMapY
        End If
    Else
        x = myMapX: y = myMapY
    End If
End Sub


Public Sub NextMaxStep(ByRef x As Integer, ByRef y As Integer)
    Dim XDir As Integer, YDir As Integer, c As Integer
    Dim CreatureX As Creature
    If myMaxStep > 0 Then
        c = 0
        XDir = myStepX(myMaxStep) - myMapX
        YDir = myStepY(myMaxStep) - myMapY
        For Each CreatureX In myCreatures
            CreatureX.MapX = myStepX(myMaxStep): CreatureX.MapY = myStepY(myMaxStep)
        Next CreatureX
        x = myStepX(myMaxStep): y = myStepY(myMaxStep)
        myMaxStep = myMaxStep - 1
    Else
        x = myMapX: y = myMapY
    End If
End Sub

Public Sub NextStepIs(ByRef x As Integer, ByRef y As Integer)
    If myMaxStep > 0 Then
        x = myStepX(myMaxStep): y = myStepY(myMaxStep)
    Else
        x = myMapX: y = myMapY
    End If
End Sub

Public Property Get MoveToX() As Integer
    MoveToX = myMoveToX
End Property

Public Property Let MoveToX(NewValue As Integer)
    myMaxStep = 0
    ReDim myStepX(0) As Integer
    ReDim myStepY(0) As Integer
    myMoveToX = NewValue
End Property

Public Property Get MoveToY() As Integer
    MoveToY = myMoveToY
End Property

Public Property Let MoveToY(NewValue As Integer)
    myMaxStep = 0
    ReDim myStepX(0) As Integer
    ReDim myStepY(0) As Integer
    myMoveToY = NewValue
End Property

Public Property Get PartySize() As Integer
    PartySize = myPartySize
End Property

Public Property Let PartySize(NewValue As Integer)
    myPartySize = NewValue
End Property

Public Property Get PartyAvgLevel() As Integer
    PartyAvgLevel = myPartyAvgLevel
End Property

Public Property Let PartyAvgLevel(NewValue As Integer)
    myPartyAvgLevel = NewValue
End Property

Public Property Get Turn() As Byte
    Turn = myTurn
End Property

Public Property Let Turn(NewTurn As Byte)
    myTurn = NewTurn
End Property

Public Property Get Cycle() As Byte
    Cycle = myCycle
End Property

Public Property Let Cycle(NewCycle As Byte)
    myCycle = NewCycle
End Property

Public Property Get Moon() As Byte
    Moon = myMoon
End Property

Public Property Let Moon(NewMoon As Byte)
    myMoon = NewMoon
End Property

Public Property Get Year() As Integer
    Year = myYear
End Property

Public Property Let Year(NewYear As Integer)
    myYear = NewYear
End Property

Public Property Get RealSeconds() As Integer
    RealSeconds = myRealSeconds
End Property

Public Property Let RealSeconds(NewRealSeconds As Integer)
    myRealSeconds = NewRealSeconds
End Property

Public Property Get RealMinutes() As Integer
    RealMinutes = myRealMinutes
End Property

Public Property Let RealMinutes(NewRealMinutes As Integer)
    myRealMinutes = NewRealMinutes
End Property

Public Property Get RealHours() As Integer
    RealHours = myRealHours
End Property

Public Property Let RealHours(NewRealHours As Integer)
    myRealHours = NewRealHours
End Property

Public Property Get RealDays() As Integer
    RealDays = myRealDays
End Property

Public Property Let RealDays(NewRealDays As Integer)
    myRealDays = NewRealDays
End Property

Public Property Get Version() As Integer
    Version = myVersion
End Property

Public Property Let Version(NewValue As Integer)
    myVersion = NewValue
End Property

Public Property Let OnAdventure(NewValue As Integer)
    If NewValue Then
        myOnAdventure = myOnAdventure Or &H1
    Else
        myOnAdventure = myOnAdventure And (Not &H1)
    End If
End Property

Public Property Get IsReset() As Integer
    IsReset = (myOnAdventure And &H2) > 0
End Property

Public Property Let IsReset(NewValue As Integer)
    If NewValue Then
        myOnAdventure = myOnAdventure Or &H2
    Else
        myOnAdventure = myOnAdventure And (Not &H2)
    End If
End Property

Public Property Get IsInPlay() As Integer
    IsInPlay = (myOnAdventure And &H4) > 0
End Property

Public Property Let IsInPlay(NewValue As Integer)
    If NewValue Then
        myOnAdventure = myOnAdventure Or &H4
    Else
        myOnAdventure = myOnAdventure And (Not &H4)
    End If
End Property

Public Property Get OnAdventure() As Integer
    OnAdventure = (myOnAdventure And &H1) > 0
End Property

Public Property Get Triggers() As Collection
    Set Triggers = myTriggers
End Property

Public Function AddTrigger() As Trigger
    ' Adds a Trigger and returns a reference to that Trigger
    Dim c As Integer
    Dim TriggerX As Trigger
    ' Find new available unused index idenifier
    c = 1
    For Each TriggerX In myTriggers
        If TriggerX.Index >= c Then
            c = TriggerX.Index + 1
        End If
    Next TriggerX
    ' Create the Trigger, set the Index and add it.
    Set TriggerX = New Trigger
    TriggerX.Index = c
    TriggerX.Name = "Trigger" & c
    myTriggers.Add Item:=TriggerX, Key:="T" & TriggerX.Index
    Set AddTrigger = TriggerX
End Function

Public Sub RemoveTrigger(DeleteKey As String)
    myTriggers.Remove DeleteKey
End Sub

Public Property Get Creatures() As Collection
    Set Creatures = myCreatures
End Property

Public Function AddCreature() As Creature
    ' Adds a Trigger and returns a reference to that Trigger
    Dim c As Integer
    Dim CreatureX As Creature
    ' Find new available unused index idenifier
    c = 1
    For Each CreatureX In myCreatures
        If CreatureX.Index >= c Then
            c = CreatureX.Index + 1
        End If
    Next CreatureX
    ' Create the Trigger, set the Index and add it.
    Set CreatureX = New Creature
    CreatureX.Index = c
    CreatureX.Name = "Creature" & c
    myCreatures.Add Item:=CreatureX, Key:="X" & CreatureX.Index
    Set AddCreature = CreatureX
End Function

Public Sub AppendCreature(CreatureX As Creature)
    ' Adds a Trigger and returns a reference to that Trigger
    Dim c As Integer
    Dim CreatureZ As Creature
    ' Find new available unused index idenifier
    c = 1
    For Each CreatureZ In myCreatures
        If CreatureZ.Index >= c Then
            c = CreatureZ.Index + 1
        End If
    Next CreatureZ
    ' Create the Trigger, set the Index and add it.
    CreatureX.Index = c
    myCreatures.Add Item:=CreatureX, Key:="X" & CreatureX.Index
End Sub

Public Sub RemoveCreature(DeleteKey As String)
    myCreatures.Remove DeleteKey
End Sub

Public Property Get Factoids() As Collection
    Set Factoids = myFactoids
End Property

Public Function AddFactoid() As Factoid
    ' Adds a Factoid and returns a reference to that Factoid
    Dim c As Integer
    Dim FactoidX As Factoid
    ' Find new available unused index idenifier
    c = 1
    For Each FactoidX In myFactoids
        If FactoidX.Index >= c Then
            c = FactoidX.Index + 1
        End If
    Next FactoidX
    ' Create the Factoid, set the Index and add it.
    Set FactoidX = New Factoid
    FactoidX.Index = c
    myFactoids.Add Item:=FactoidX, Key:="F" & FactoidX.Index
    Set AddFactoid = FactoidX
End Function

Public Sub RemoveFactoid(DeleteKey As String)
    myFactoids.Remove DeleteKey
End Sub

Public Property Get Journals() As Collection
    Set Journals = myJournals
End Property

Public Function AddJournal() As Journal
    ' Adds a Journal and returns a reference to that Journal
    Dim c As Integer
    Dim JournalX As Journal
    ' Find new available unused index idenifier
    c = 1
    For Each JournalX In myJournals
        If JournalX.Index >= c Then
            c = JournalX.Index + 1
        End If
    Next JournalX
    ' Create the Journal, set the Index and add it.
    Set JournalX = New Journal
    JournalX.Index = c
    myJournals.Add Item:=JournalX, Key:="J" & JournalX.Index
    Set AddJournal = JournalX
End Function

Public Sub RemoveJournal(DeleteKey As String)
    myJournals.Remove DeleteKey
End Sub

Public Property Get Areas() As Collection
    Set Areas = myAreas
End Property

Public Function AddArea() As Area
    ' Adds a Area and returns a reference to that Area
    Dim c As Integer
    Dim AreaX As Area
    ' Find new available unused index idenifier
    c = 1
    For Each AreaX In myAreas
        If AreaX.Index >= c Then
            c = AreaX.Index + 1
        End If
    Next AreaX
    ' Create the Area, set the Index and add it.
    Set AreaX = New Area
    AreaX.Index = c
    AreaX.Name = "Area" & AreaX.Index
    myAreas.Add Item:=AreaX, Key:="A" & AreaX.Index
    Set AddArea = AreaX
End Function

Public Sub RemoveArea(DeleteKey As String)
    myAreas.Remove DeleteKey
End Sub

Public Sub ReadFromFileHeader(FromFile As Integer)
    Dim c As Integer, Cnt As Long
    ' Read Item Name, Index and Comments
    Get FromFile, , myVersion
    Get FromFile, , Cnt
    myName = ""
    For c = 1 To Cnt
        myName = myName & " "
    Next c
    Get FromFile, , myName
    ' Read Comments
    Get FromFile, , Cnt
    myComments = ""
    For c = 1 To Cnt
        myComments = myComments & " "
    Next c
    Get FromFile, , myComments
    If Len(BreakText(myComments, 2)) > 0 Then
        myRealSeconds = BreakText(myComments, 2)
        myRealMinutes = BreakText(myComments, 3)
        myRealHours = BreakText(myComments, 4)
        myRealDays = BreakText(myComments, 5)
    Else
        myRealSeconds = 0
        myRealMinutes = 0
        myRealHours = 0
        myRealDays = 0
    End If
    myComments = BreakText(myComments, 1)
    ' Read FullPath
    Get FromFile, , Cnt
    myFullPath = ""
    For c = 1 To Cnt
        myFullPath = myFullPath & " "
    Next c
    Get FromFile, , myFullPath
    ' Read for Maps
    Get FromFile, , myAreaIndex
    Get FromFile, , myMapIndex
    Get FromFile, , myEntryIndex
    Get FromFile, , myMapX
    Get FromFile, , myMapY
    Get FromFile, , myTurn
    Get FromFile, , myCycle
    Get FromFile, , myMoon
    Get FromFile, , myYear
    Get FromFile, , myPartySize
    Get FromFile, , myPartyAvgLevel
    Get FromFile, , myOnAdventure
End Sub

Public Sub ReadFromFile(FromFile As Integer)
    Dim c As Integer, Cnt As Long
    Dim myTrigger As Trigger, myCreature As Creature
    Dim myFactoid As Factoid, myJournal As Journal, myArea As Area
    ' Read header information
    ReadFromFileHeader FromFile
    ' Read Party Triggers for Tome
    Get FromFile, , Cnt
    For c = 1 To Cnt
        Set myTrigger = New Trigger
        myTrigger.ReadFromFile FromFile
        myTriggers.Add Item:=myTrigger, Key:="T" & myTrigger.Index
    Next c
    ' Read Party Members for Tome
    Get FromFile, , Cnt
    For c = 1 To Cnt
        Set myCreature = New Creature
        myCreature.ReadFromFile FromFile
        myCreatures.Add Item:=myCreature, Key:="X" & myCreature.Index
    Next c
    ' Read Party Journals for Tome
    Get FromFile, , Cnt
    For c = 1 To Cnt
        Set myJournal = New Journal
        myJournal.ReadFromFile FromFile
        myJournals.Add Item:=myJournal, Key:="J" & myJournal.Index
    Next c
    ' Read Party Factoids for Tome
    Get FromFile, , Cnt
    For c = 1 To Cnt
        Set myFactoid = New Factoid
        myFactoid.ReadFromFile FromFile
        myFactoids.Add Item:=myFactoid, Key:="F" & myFactoid.Index
    Next c
    ' Read Party Areas for Tome
    Get FromFile, , Cnt
    For c = 1 To Cnt
        Set myArea = New Area
        myArea.ReadFromFile FromFile
        myAreas.Add Item:=myArea, Key:="A" & myArea.Index
    Next c
    myDirty = False
End Sub

Public Sub SaveToFile(ToFile As Integer)
    Dim c As Integer, tmp As String
    ' Save Item Name, Index and Comments
    Put ToFile, , myVersion
    Put ToFile, , Len(myName)
    Put ToFile, , myName
    If myRealSeconds > 0 Or myRealMinutes > 0 Or myRealHours > 0 Or myRealDays > 0 Then
        tmp = myComments & "|" & myRealSeconds & "|" & myRealMinutes & "|" & myRealHours & "|" & myRealDays
    Else
        tmp = myComments
    End If
    Put ToFile, , Len(tmp)
    Put ToFile, , tmp
    Put ToFile, , Len(myFullPath)
    Put ToFile, , myFullPath
    ' Save PlotFile
    Put ToFile, , myAreaIndex
    Put ToFile, , myMapIndex
    Put ToFile, , myEntryIndex
    Put ToFile, , myMapX
    Put ToFile, , myMapY
    Put ToFile, , myTurn
    Put ToFile, , myCycle
    Put ToFile, , myMoon
    Put ToFile, , myYear
    Put ToFile, , myPartySize
    Put ToFile, , myPartyAvgLevel
    Put ToFile, , myOnAdventure
    ' Save Triggers
    Put ToFile, , myTriggers.Count
    For c = 1 To myTriggers.Count
        myTriggers(c).SaveToFile ToFile
    Next c
    ' Save Party
    Put ToFile, , myCreatures.Count
    For c = 1 To myCreatures.Count
        myCreatures(c).SaveToFile ToFile
    Next c
    ' Save Journals
    Put ToFile, , myJournals.Count
    For c = 1 To myJournals.Count
        myJournals(c).SaveToFile ToFile
    Next c
    ' Save Factoids
    Put ToFile, , myFactoids.Count
    For c = 1 To myFactoids.Count
        myFactoids(c).SaveToFile ToFile
    Next c
    ' Save Areas
    Put ToFile, , myAreas.Count
    For c = 1 To myAreas.Count
        myAreas(c).SaveToFile ToFile
    Next c
End Sub

Private Sub Class_Initialize()
    myName = "Untitled"
    myLoadPath = "Not Saved"
    myTurn = 100
    myCycle = 25
    myMoon = 5
    myYear = 5000
    myPartyAvgLevel = 1
    myPartySize = 5
End Sub

Public Property Get Dirty() As Integer
    Dirty = myDirty
End Property

